//+------------------------------------------------------------------+
//|                                        wpr + bollinger bands.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     DeepSkyBlue
#property indicator_color2     Snow
#property indicator_color3     DarkOrange
#property indicator_color4     Snow
#property indicator_style2     STYLE_DASH
#property indicator_style3     STYLE_DASH
#property indicator_style4     STYLE_DASH
#property indicator_width1     2
#property indicator_maximum    120
#property indicator_minimum    -20
#property indicator_level1     20
#property indicator_level2     80
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor Magenta
#property strict

//
//
//
//
//

extern string TimeFrame        = "Current time frame";
extern int    HLR_Range        = 100;
extern int    BandsPeriod      = 50;
extern double BandsDeviations  = 2.2;
extern bool   Interpolate      = true;

extern string  __              = "arrows settings";
extern bool   ShowArrows       = false; 
extern string arrowsIdentifier = "rosc arrows";
extern color  arrowsUpColor    = Lime;
extern color  arrowsDnColor    = Orange;

//
//
//
//
//

double HLR[];
double bandsUp[];
double bandsMi[];
double bandsDo[];
double trend[];

//
//
//
//
//

string indicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,HLR);
   SetIndexBuffer(1,bandsUp);
   SetIndexBuffer(2,bandsMi);
   SetIndexBuffer(3,bandsDo);
   SetIndexBuffer(4,trend);

      //
      //
      //
      //
      //
      
      indicatorFileName = WindowExpertName();
      calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //
      
   IndicatorShortName(timeFrameToString(timeFrame)+" Range osc with bands ("+(string)HLR_Range+","+(string)BandsPeriod+","+DoubleToStr(BandsDeviations,2)+")");
   return(0);
}

int deinit()
{
   if (!calculateValue && ShowArrows) deleteArrows();
   
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { HLR[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
     for (i=limit-1; i>=0; i--)
     {  
        double hhv = iHigh(NULL,0,iHighest(NULL,0,MODE_HIGH,HLR_Range,i));
        double llv =  iLow(NULL,0,iLowest( NULL,0, MODE_LOW,HLR_Range,i)); 
        double mpr = (iHigh(NULL,0,i) + iLow(NULL,0,i)) / 2.0; 
        if(hhv!=llv)
             HLR[i] = 100.0 * (mpr-llv)/(hhv-llv); 
        else HLR[i] = 0.0; 
      }
      
      //
      //
      //
      //
      //
      
      for(i=limit; i>=0; i--) 
      {
         double deviations = iStdDevOnArray(HLR,0,BandsPeriod,0,MODE_LWMA,i);
         bandsMi[i] = iMAOnArray(HLR,0,BandsPeriod,0,MODE_LWMA,i);
         bandsUp[i] = bandsMi[i]+deviations*BandsDeviations;
         bandsDo[i] = bandsMi[i]-deviations*BandsDeviations;
         if (i<Bars-1)
         {
            trend[i] = trend[i+1];
               if(HLR[i]<bandsDo[i]) trend[i] = 1;
               if(HLR[i]>bandsUp[i]) trend[i] =-1;
               manageArrow(i);     
         }               
       }
    return(0);
    }
     
    //
    //
    //
    //
    //
  
    limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
    for (i=limit;i>=0; i--)
    {
       int y = iBarShift(NULL,timeFrame,Time[i]);
          HLR[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HLR_Range,BandsPeriod,BandsDeviations,0,y);
          bandsUp[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HLR_Range,BandsPeriod,BandsDeviations,1,y);
          bandsMi[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HLR_Range,BandsPeriod,BandsDeviations,2,y);
          bandsDo[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HLR_Range,BandsPeriod,BandsDeviations,3,y);
          trend[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HLR_Range,BandsPeriod,BandsDeviations,4,y);
          manageArrow(i);

          //
          //
          //
          //
          //
            
          if (!Interpolate || (i!=0 && y==iBarShift(NULL,timeFrame,Time[i-1]))) continue;

          //
          //
          //
          //
          //

          int n,j;
          datetime time = iTime(NULL,timeFrame,y);
          for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
          for(j = 1; j < n && (i+j)<Bars && (i+n)<Bars; j++)
          {
  	          HLR[i+j]     = HLR[i]     + (HLR[i+n]     - HLR[i])     * j/n;
  	          bandsUp[i+j] = bandsUp[i] + (bandsUp[i+n] - bandsUp[i]) * j/n;
  	          bandsMi[i+j] = bandsMi[i] + (bandsMi[i+n] - bandsMi[i]) * j/n;
  	          bandsDo[i+j] = bandsDo[i] + (bandsDo[i+n] - bandsDo[i]) * j/n;
          }  	            
   } 
return(0);
}

//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+(string)iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                              return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str) { string s = str; StringToUpper(s); return(s); }

//
//
//
//
//

void manageArrow(int i)
{
   if (!calculateValue && ShowArrows)
   {
      deleteArrow(Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,241,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,242,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
  
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+(string)time; ObjectDelete(lookFor);
}
